#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

Memimage *
flip(Memimage *mi, int xaxis, int yaxis)
{
	Memimage *nmi;
	int i, j;
	Rectangle dstr;

	nmi = allocmemimage(mi->r, mi->chan);
	if(!nmi)
		sysfatal("allocmemimage: %r");

	for(i=mi->r.min.x; i < mi->r.max.x; i++)
		for(j=mi->r.min.y; j < mi->r.max.y; j++){
			dstr.min = Pt(yaxis ? mi->r.max.x-i-1 : i, xaxis ? mi->r.max.y-j-1 : j);
			dstr.max = addpt(dstr.min, Pt(1, 1));
			memimagedraw(nmi, dstr, mi, Pt(i, j), nil, Pt(0, 0), SoverD);
		}

	return nmi;
}

void
usage(void)
{
	fprint(2, "usage: flip [-xy]\n");
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Memimage *mi, *nmi;
	int infd, outfd;
	int axis = 0;

	ARGBEGIN {
	case 'x':
		axis |= 1;
		break;
	case 'y':
		axis |= 2;
		break;
	default:
		usage();
	} ARGEND

	if(argc != 0)
		usage();

	infd = 0;
	outfd = 1;

	memimageinit();

	mi = readmemimage(infd);
	if(!mi)
		sysfatal("reading image: %r");

	nmi = flip(mi, axis&1, axis&2);
	if(!nmi)
		sysfatal("creating new image: %r");

	if(writememimage(outfd, nmi) != 0)
		sysfatal("writing image: %r");
	exits(0);
}
