#include <u.h>
#include <libc.h>
#include <draw.h>
#include <memdraw.h>

Memimage *
rotate(Memimage *mi, int degrees)
{
	Memimage *nmi;
	int i, j;
	Rectangle newr;

	newr = mi->r;
	if(degrees == 90 || degrees == 270)
		newr = Rect(mi->r.min.y, mi->r.min.x, mi->r.max.y, mi->r.max.x);
	nmi = allocmemimage(newr, mi->chan);
	if(!nmi)
		sysfatal("allocmemimage: %r");

	for(i=mi->r.min.x; i < mi->r.max.x; i++)
		for(j=mi->r.min.y; j < mi->r.max.y; j++){
			if(degrees == 90)
				memimagedraw(nmi, Rect(nmi->r.max.x-j-1, i, nmi->r.max.x-j-1+1, i+1), mi, Pt(i, j), nil, Pt(0, 0), SoverD);
			else if(degrees == 180)
				memimagedraw(nmi, Rect(nmi->r.max.x-i-1, nmi->r.max.y-j-1, nmi->r.max.x-i, nmi->r.max.y-j), mi, Pt(i, j), nil, Pt(0, 0), SoverD);
			else if(degrees == 270)
				memimagedraw(nmi, Rect(j, nmi->r.max.y-i-1, j+1, nmi->r.max.y-i), mi, Pt(i, j), nil, Pt(0, 0), SoverD);
			else
				werrstr("invalid degrees (%d), can only rotate by 90, 180, 270", degrees);
		}

	return nmi;
}

void
usage(void)
{
	fprint(2, "usage: rotate [90 | 180 | 270]\n");
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Memimage *mi, *nmi;
	int infd, outfd;
	int degrees;

	ARGBEGIN {
	default:
		usage();
	} ARGEND

	if(argc != 1)
		usage();

	degrees = atoi(argv[0]);
	switch(degrees){
	case 90:
	case 180:
	case 270:
		break;
	default:
		usage();
	}

	infd = 0;
	outfd = 1;

	memimageinit();

	mi = readmemimage(infd);
	if(!mi)
		sysfatal("reading image: %r");

	nmi = rotate(mi, degrees);
	if(!nmi)
		sysfatal("creating new image: %r");

	if(writememimage(outfd, nmi) != 0)
		sysfatal("writing image: %r");
	exits(0);
}
